var BOSH_SERVICE = 'http://localhost:5280/http-bind/';

Strophe.addNamespace("ADHOC", "http://jabber.org/protocol/commands");

var localJID = null;
var connection   = null;
var sessionid = null;
var cmdNode = null;

function log(msg) {
    var entry = $('<div></div>').append(document.createTextNode(msg));
    $('#log').append(entry);
}

function rawInput(data) {
    log('RECV: ' + data);
}

function rawOutput(data) {
    log('SENT: ' + data);
}

function onConnect(status) {
    if (status == Strophe.Status.CONNECTING) {
        log('Strophe is connecting.');
    } else if (status == Strophe.Status.CONNFAIL) {
        log('Strophe failed to connect.');
        showConnect();
    } else if (status == Strophe.Status.DISCONNECTING) {
        log('Strophe is disconnecting.');
    } else if (status == Strophe.Status.DISCONNECTED) {
        log('Strophe is disconnected.');
        showConnect();
    } else if (status == Strophe.Status.AUTHFAIL) {
        log('Authentication failed');
        if (connection) {
            connection.disconnect();
        }
    } else if (status == Strophe.Status.CONNECTED) {
        log('Strophe is connected.');
        checkFeatures();
    }
}

function displayResult(result) {
    var status = $(result).find("command").attr("status");

    $("#output *").remove();
    $(result).find("note").each(function(index, e) {
        var type = $(e).attr("type");
        if (!type) {
           type = "info";
        }
        $("#output").append("<p class='" + type + "Note'>" + $(e).text() + "</p>");
    });
    if (status == "executing") {
        $("#output").append("<input type='button' disabled='true' id='prevButton' value='Prev'/>"+
                            "<input type='button' disabled='true' id='nextButton' value='Next'/>"+
                            "<input type='button' disabled='true' id='completeButton' value='Complete'/>"+
                            "<input type='button' id='executeButton' value='Execute'/>"+
                            "<input type='button' id='cancelButton' value='Cancel'/>");
        for (kind in ['prev', 'next', 'complete']) {
            if ($(result).find('actions ' + kind).length > 0)
                $('#' + kind + 'Button').attr("disabled", "false");
        }
        $('#cancelButton').bind("click", function() {
            var cancelIQ = $iq({ type: "set", to: "localhost", id: connection.getUniqueId() })
                .c("command", { xmlns: Strophe.NS.ADHOC, node: cmdNode, sessionid: sessionid, action: "cancel" });
            cmdNode = null
            sessionid = null;
            connection.sendIQ(cancelIQ, displayResult);
        });
    } else {
        input = $("<input type='button' value='Restart'/>").bind("click", function() {
            $('#output *').remove();
            sessionid = null;
            cmdNode = null;
            getCommandNodes();
        });
        $("#output").append(input);
    }
}

function runCommand() {
    cmdNode = $(this).attr("id"); // Save not of executed command (in global)
    var execIQ = $iq({ type: "set", to: "localhost", id: connection.getUniqueId() })
        .c("command", { xmlns: Strophe.NS.ADHOC, node: cmdNode, action: "execute" });
    connection.sendIQ(execIQ, function(result) {
        sessionid = $(result).find("command").attr("sessionid");
        displayResult(result);
    });
}

function getCommandNodes() {
    var nodesIQ = $iq({ type: "get", to: "localhost", id: "nodes1" }).c("query", {xmlns: Strophe.NS.DISCO_ITEMS, node: Strophe.NS.ADHOC});
    connection.sendIQ(nodesIQ, function(result) {
        $('#output').append("<ul id='items'></ul>");
        $(result).find("item").each(function(index, e) {
            item = $("<li id='" + $(e).attr("node") + "'>" + $(e).attr("name") + "</li>").bind("click", runCommand);
            $("#items").append(item);
        });
    });
}

function checkFeatures() {
    featureIQ = $iq({ type: "get", to: "localhost", id: "features1" }).c("query", {xmlns: Strophe.NS.DISCO_INFO});
    connection.sendIQ(featureIQ, function(result) {
        if ($(result).find("feature[var='" + Strophe.NS.ADHOC + "']").length > 0) {
            $('#output').append("<p>This entitiy does support AdHoc commands</p>");
        } else {
            $('#output').append("<p>This entitiy does NOT support AdHoc commands</p>");
        }
    });
    getCommandNodes();
}

function showConnect() {
    var jid = $('#jid');
    var pass = $('#pass');
    var button = $('#connect').get(0);        

    button.value = 'connect';
    pass.show();
    jid.show();
    $('label').show();
    $('#output *').remove();
    return false;
}

function showDisconnect() {
    var jid = $('#jid');
    var pass = $('#pass');
    var button = $('#connect').get(0);        

    button.value = 'disconnect';
    pass.hide();
    jid.hide();
    $('label').hide();
    return false;
}

$(document).ready(function () {
    connection = new Strophe.Connection(BOSH_SERVICE);
    connection.rawInput = rawInput;
    connection.rawOutput = rawOutput;

    $("#log_toggle").click(function () {
        $("#log").toggle();
      });

    $('#cred').bind('submit', function () {
        var button = $('#connect').get(0);
        var jid = $('#jid');
        var pass = $('#pass');        
        localJID = jid.get(0).value;

        if (button.value == 'connect') {
            showDisconnect();
            $('#log *').remove();
            connection.connect(localJID,
               pass.get(0).value,
               onConnect);
        } else {
            connection.disconnect();
        }
        return false;
    });
});

onunload = function() {
    if (connection) {
        connection.disconnect();
    }
}

